#Region "Copyright (C) 2013 James D Westoby"
'  
'  *	Copyright (C) 2013 James D Westoby
'  *
'  *  This Program is free software you can redistribute it and/or modify
'  *  it under the terms of the GNU General Public License as published by
'  *  the Free Software Foundation either version 2, or (at your option)
'  *  any later version.
'  *   
'  *  This Program is distributed in the hope that it will be useful,
'  *  but WITHOUT ANY WARRANTY without even the implied warranty of
'  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'  *  GNU General Public License for more details.
'  *   
'  *  You should have received a copy of the GNU General Public License
'  *  along with GNU Make see the file COPYING.  If not, write to
'  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
'  *  http:'www.gnu.org/copyleft/gpl.html
'  *
'  */


#End Region

Imports System
Imports System.Text

Imports MediaPortal.Core
Imports MediaPortal.Configuration
Imports MediaPortal.Dialogs
Imports MediaPortal.GUI.Library


 
Namespace ClockPlugin
    Public Class ClockCommon
        Public Const ConfigForm As Integer = 6868
        Public Const ConfigSkin As String = "\ClockConfig.xml"

        Public Const MainForm As Integer = 6867
        Public Const MainSkin As String = "\ClockMain.xml"

        Private Shared myConfig As String = "Clock.xml"  'Config filename

        Private myID As Integer                         'parent window, main or config
        Private mySettings As MediaPortal.Profile.Settings


        Private myChime As Boolean
        Private myDate As String
        Private myMeridian As Boolean
        Private myTime As String
        Private myTimeoutBright As Integer = 60
        Private myTimeoutDim As Integer = 5


        'constructors/destructor

        Public Sub New(ByVal WID As Integer)
            myID = WID
            mySettings = _
                New MediaPortal.Profile.Settings(MediaPortal.Configuration.Config.GetFile(MediaPortal.Configuration.Config.Dir.Config, myConfig))
            ConfigLoad()
        End Sub


        Protected Overrides Sub Finalize()
            mySettings.Dispose()
            MyBase.Finalize()
        End Sub


        'public methods

        Public Sub ConfigLoad()
            myChime = mySettings.GetValueAsBool("Clock", "Chime", False)
            myDate = mySettings.GetValueAsString("Clock", "Date", myDate)
            myMeridian = mySettings.GetValueAsBool("Clock", "Meridian", False)
            myTimeoutBright = mySettings.GetValueAsInt("Clock", "TimeoutBright", myTimeoutBright)
            myTimeoutDim = mySettings.GetValueAsInt("Clock", "TimeoutDim", myTimeoutDim)
            myTime = mySettings.GetValueAsString("Clock", "Time", myTime)
        End Sub


        Public Sub ConfigSave()
            mySettings.SetValue("Clock", "Chime", "" & myChime)
            mySettings.SetValue("Clock", "Date", myDate)
            mySettings.SetValue("Clock", "Meridian", "" & myMeridian)
            mySettings.SetValue("Clock", "Time", myTime)
            mySettings.SetValue("Clock", "TimeoutBright", myTimeoutBright)
            mySettings.SetValue("Clock", "TimeoutDim", myTimeoutDim)
        End Sub


        Public Function EnterRequest(ByVal Initial As String) As String
            'get info from the virtual keyboard, if not just send back the original string
            Dim k As VirtualKeyboard = CType(GUIWindowManager.GetWindow(CType(GUIWindow.Window.WINDOW_VIRTUAL_KEYBOARD, Integer)), VirtualKeyboard)
            If k Is Nothing Then Return ("")

            k.Reset()
            k.SetLabelAsInitialText(False)
            k.Text = Initial
            k.DoModal(myID)
            If (k.IsConfirmed) Then Return (k.Text) Else Return (Initial)
        End Function


        Public Property Chime() As Boolean
            Get
                Return (myChime)
            End Get
            Set(ByVal Value As Boolean)
                myChime = Value
            End Set
        End Property


        Public Property DateFormat() As String
            Get
                Return (myDate)
            End Get
            Set(ByVal Value As String)
                myDate = Value
            End Set
        End Property


        Public Property Meridian() As Boolean
            Get
                Return (myMeridian)
            End Get
            Set(ByVal Value As Boolean)
                myMeridian = Value
            End Set
        End Property


        Public ReadOnly Property Text(ByVal Val As String) As String
            Get
                Dim v As String = mySettings.GetValue("Translate", Val)
                Return (CType(IIf((v = ""), Val, v), String))
            End Get
        End Property


        Public Property TimeoutBright() As Integer
            Get
                Return (myTimeoutBright)
            End Get
            Set(ByVal Value As Integer)
                Try
                    myTimeoutBright = CInt(Value)
                Catch ex As Exception
                    'donothing
                End Try
            End Set
        End Property


        Public Property TimeoutDim() As Integer
            Get
                Return (myTimeoutDim)
            End Get
            Set(ByVal Value As Integer)
                Try
                    myTimeoutDim = CInt(Value)
                Catch ex As Exception
                    'donothing
                End Try
            End Set
        End Property


        Public Property TimeFormat() As String
            Get
                Return (myTime)
            End Get
            Set(ByVal Value As String)
                myTime = Value
            End Set
        End Property

    End Class

End Namespace
