#Region "Copyright (C) 2013 James D Westoby"

' Copyright (C) 2013 James D Westoby
'
'  This Program is free software you can redistribute it and/or modify
'  it under the terms of the GNU General Public License as published by
'  the Free Software Foundation either version 2, or (at your option)
'  any later version.
'   
'  This Program is distributed in the hope that it will be useful,
'  but WITHOUT ANY WARRANTY without even the implied warranty of
'  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'  GNU General Public License for more details.
'   
'  You should have received a copy of the GNU General Public License
'  along with GNU Make see the file COPYING.  If not, write to
'  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
'  http:'www.gnu.org/copyleft/gpl.html

#End Region


Imports System
Imports System.Diagnostics.Process
Imports System.Timers
Imports System.Runtime.InteropServices

Imports MediaPortal.GUI.Library
Imports MediaPortal.Util


Namespace ClockPlugin
    Public Class ClockPlugin
        Inherits GUIWindow
        Implements ISetupForm

        Private Declare Function FindWindow Lib "user32" Alias _
        "FindWindow" (ByVal lpClassName As String, ByVal lpWindowName As String) As IntPtr

        Private Declare Function GetForegroundWindow Lib "user32" Alias _
        "GetForegroundWindow" () As IntPtr

        Private Declare Function SendMessage Lib "user32" Alias _
        "SendMessageA" (ByVal hWnd As IntPtr, ByVal wMsg As Integer, _
                        ByVal wParam As Integer, ByVal lParam As Integer) As Integer
        Private Const WM_SYSCOMMAND As Integer = &H112
        Private Const SC_MONITORPOWER As Integer = &HF170
        Private Const MONITOR_OFF As Integer = 2
        Private Const MONITOR_ON As Integer = -1

        Private Declare Function SetForegroundWindow Lib "user32" Alias _
            "SetForegroundWindow" (ByVal hWnd As IntPtr) As Boolean

        <SkinControl(11)> Protected buttonDim As GUIButtonControl = Nothing

        <SkinControl(21)> Protected labelTime As GUILabelControl = Nothing
        <SkinControl(22)> Protected labelDate As GUILabelControl = Nothing
        <SkinControl(31)> Protected labelMeridian As GUILabelControl = Nothing

        '<SkinControl(39)> Protected labelInfo As GUILabelControl = Nothing

        Private myCommon As ClockCommon = Nothing

        Private myMP As IntPtr = IntPtr.Zero

        Private Shared mySelf As ClockPlugin = Nothing
        Private Shared ourTimer As System.Timers.Timer = Nothing

        Private myTimeoutTx As Integer = -1             'transfer timeout value to thread
        Private myTimeout As Integer = -1               'thread timeout counter (zero is changed to -1 as no timeout)

        Private myScreenLevel As String = "Off"         'Off, Dim or Bright



#Region "ISetupFormMembers"

        Public Function Author1() As String Implements MediaPortal.GUI.Library.ISetupForm.Author
            Return ("James D Westoby")
        End Function


        Public Function Description() As String Implements MediaPortal.GUI.Library.ISetupForm.Description
            Return ("Clock Plugin")
        End Function


        Public Function CanEnable1() As Boolean Implements MediaPortal.GUI.Library.ISetupForm.CanEnable
            Return (True)
        End Function


        Public Function DefaultEnabled() As Boolean Implements MediaPortal.GUI.Library.ISetupForm.DefaultEnabled
            Return (True)
        End Function


        Public Function GetHome(ByRef strButtonText As String, ByRef strButtonImage As String, ByRef strButtonImageFocus As String, ByRef strPictureImage As String) As Boolean Implements MediaPortal.GUI.Library.ISetupForm.GetHome
            strButtonText = PluginName()
            strButtonImage = String.Empty
            strButtonImageFocus = String.Empty
            strButtonImage = String.Empty
            strPictureImage = "hover_clock.png"
            Return (True)
        End Function


        Public Overrides Property GetID() As Integer
            Get
                Return (GetWindowId())
            End Get
            Set(ByVal value As Integer)
                'donothing
            End Set
        End Property


        Public Function GetWindowId() As Integer Implements MediaPortal.GUI.Library.ISetupForm.GetWindowId
            Return (ClockCommon.MainForm)
        End Function


        Public Function HasSetup() As Boolean Implements MediaPortal.GUI.Library.ISetupForm.HasSetup
            Return (True)
        End Function


        Public Function PluginName() As String Implements MediaPortal.GUI.Library.ISetupForm.PluginName
            Return ("Clock")
        End Function


        Public Sub ShowPlugin() Implements MediaPortal.GUI.Library.ISetupForm.ShowPlugin
            'configure inside MediaPortal
        End Sub

#End Region


#Region "ClockPlugin class"

        Public Overrides Function Init() As Boolean
            mySelf = Me

            myCommon = New ClockCommon(GetWindowId())
            ourTimer = New System.Timers.Timer()
            ourTimer.Enabled = False
            ourTimer.AutoReset = False
            AddHandler ourTimer.Elapsed, New ElapsedEventHandler(AddressOf TickTock)

            Return (Load(GUIGraphicsContext.Skin & ClockCommon.MainSkin))
        End Function


        Public Overrides Sub OnAction(ByVal action As MediaPortal.GUI.Library.Action)
            Select Case (myScreenLevel)
                Case "Bright" : 'donothing
                Case "Dim" :    'donothing
                Case "Off" :    SetScreen("Dim")
            End Select
            MyBase.OnAction(action)     'do whatever was requested
        End Sub


        Protected Overrides Sub OnClicked(ByVal controlId As Integer, ByVal control As GUIControl, ByVal actionType As MediaPortal.GUI.Library.Action.ActionType)
            Select Case (myScreenLevel)
                Case "Bright" : SetScreen("Dim")
                Case "Dim" :    SetScreen("Bright")
                Case "Off" :    SetScreen("Dim")
            End Select
            MyBase.OnClicked(controlId, control, actionType)
        End Sub


        Protected Overrides Sub OnMouseMove(cx As Integer, cy As Integer, action As MediaPortal.GUI.Library.Action)
            SetScreen("Bright")
        End Sub


        Protected Overrides Sub OnPageDestroy(ByVal NewWindowId As Integer)
            ourTimer.Stop()
            MyBase.OnPageDestroy(NewWindowId)       'and call the base class too
        End Sub


        Protected Overrides Sub OnPageLoad()
            myMP = GetMediaPortalWindowHandle()
            myTimeout = -1
            myTimeoutTx = -1

            buttonDim.DimColor = 0
            labelDate.DimColor = 0
            labelMeridian.DimColor = 0

            buttonDim.Selected = True
            'buttonDim.Visible = false
            myCommon.ConfigLoad()
            MyBase.OnPageLoad()

            SetDate()
            SetScreen("Dim")
            Tick()
        End Sub


        'Public Overrides Sub Render(ByVal timePassed As Single)
        'donothing
        'End Sub

        Protected Overrides Sub OnShowContextMenu()
            ourTimer.Enabled = False
            GUIWindowManager.ActivateWindow(ClockCommon.ConfigForm)
        End Sub

#End Region


#Region "Shared Methods"

        Shared Sub TickTock(ByVal Sender As Object, ByVal Tick As ElapsedEventArgs)
            ourTimer.Enabled = False
            mySelf.Tick()       'jump back into the class
        End Sub

#End Region


#Region "Private Methods"

        Private Sub BlankScreen(ByRef Mode As String)
            myScreenLevel = Mode

            MediaPortal.GUI.Library.GUIGraphicsContext.BlankScreen = (Mode = "Off")
            If (myMP <> IntPtr.Zero) Then SetForegroundWindow(myMP)

            If (Mode <> "Off") Then SetDate()

            Dim v As Integer : v = CType(IIf((Mode = "Off"), MONITOR_OFF, MONITOR_ON), Integer)
            'don't like using -1 it's a broadcast!
            SendMessage(GetForegroundWindow(), WM_SYSCOMMAND, SC_MONITORPOWER, v)
            'Dim m As GUIMessage = _
            '    New GUIMessage(GUIMessage.MessageType.PS_ONSTANDBY, 0, 0, 0, 0, 0, null)
            'GUIGraphicsContext.SendMessage(m)
        End Sub


        Private Sub DimControls(ByRef ToDim As Boolean)
            buttonDim.Label = CType(IIf(ToDim, "Bright", "Dim"), String)

            buttonDim.Dimmed = ToDim
            labelTime.Dimmed = ToDim
            labelDate.Dimmed = ToDim
            labelMeridian.Dimmed = ToDim
        End Sub


        Private Function GetMediaPortalWindowHandle() As IntPtr
            Dim p As System.Diagnostics.Process() = GetProcessesByName("MediaPortal")
            If (0 < p.Length) Then
                Return (p(0).MainWindowHandle)
            Else
                Return (FindWindow(vbNullString, "MediaPortal"))
            End If
            Return (IntPtr.Zero)
        End Function


        Private Sub SetDate()
            labelDate.Label = Format(Now, myCommon.DateFormat)
        End Sub


        Private Sub SetScreen(ByRef Mode As String)
            myScreenLevel = Mode

            Select Case (Mode)
                Case "Off"                      'only from the Tick method (so no timeout set)
                    DimControls(True)
                Case "Dim"
                    DimControls(True)
                    myTimeoutTx = myCommon.TimeoutDim
                Case "Bright"
                    DimControls(False)
                    myTimeoutTx = myCommon.TimeoutBright
            End Select
            BlankScreen(Mode)
            ourTimer.Interval = 10
            ourTimer.Enabled = (Mode <> "Off")
        End Sub


        Private Sub SetTime()
            'set the time and am/pm indicators
            labelTime.Label = Format(Now, myCommon.TimeFormat)
            If (myCommon.Meridian) Then _
                labelMeridian.Label = CType(IIf(Hour(Now) < 12, "AM", "PM"), String)

            'midnight? change the date
            If ((Now.Hour + Now.Minute) = 0) Then SetDate()
        End Sub


        Private Sub Tick()
            'labelInfo.Label = "Tick:" & myTimeout

            'new timeout value? if it's zero, flip it to -1, else just set 'as is'
            If (myTimeoutTx <> -1) Then myTimeout = CType(IIf((myTimeoutTx = 0), -1, (myTimeoutTx + 1)), Integer)
            myTimeoutTx = -1

            If (myTimeout = 0) Then Return 'no point in doing anything?

            'if it's not -1 decrement the timeout
            myTimeout = myTimeout - CType(IIf((myTimeout = -1), 0, 1), Integer)

            'timed out? blank the screen but allow this tick time/date to be set
            If (myTimeout = 0) Then SetScreen("Off")

            SetTime()                       'update the screen controls

            If (Now.Minute = 0) Then        'hour chime?
                If (myCommon.Chime) Then
                    Dim f As String = GUIGraphicsContext.Skin & "\Sounds\Chime.Wav"
                    Utils.PlaySound(f, False, True)
                End If
            End If

            'set the timer to the next minute boundary
            ourTimer.Interval = ((60 - Now.Second) * 1000)
            ourTimer.Enabled = True
        End Sub

#End Region

    End Class

End Namespace
